@@ -50,9 +50,9 @@ class SiteItem(object):
         self,
         name: str,
         content_url: str,
-        admin_mode: str = None,
-        user_quota: int = None,
-        storage_quota: int = None,
+        admin_mode: Optional[str] = None,
+        user_quota: Optional[int] = None,
+        storage_quota: Optional[int] = None,
         disable_subscriptions: bool = False,
         subscribe_others_enabled: bool = True,
         revision_history_enabled: bool = False,
