@@ -45,6 +45,9 @@ if TYPE_CHECKING:
     from ...models.connection_credentials import ConnectionCredentials
     from .schedules_endpoint import AddResponse
 
+io_types_r = (io.BytesIO, io.BufferedReader)
+io_types_w = (io.BytesIO, io.BufferedWriter)
+
 # The maximum size of a file that can be published in a single request is 64MB
 FILESIZE_LIMIT = 1024 * 1024 * 64  # 64MB
 
@@ -53,7 +56,10 @@ ALLOWED_FILE_EXTENSIONS = ["twb", "twbx"]
 logger = logging.getLogger("tableau.endpoint.workbooks")
 FilePath = Union[str, os.PathLike]
 FileObject = Union[io.BufferedReader, io.BytesIO]
-PathOrFile = Union[FilePath, FileObject]
+FileObjectR = Union[io.BufferedReader, io.BytesIO]
+FileObjectW = Union[io.BufferedWriter, io.BytesIO]
+PathOrFileR = Union[FilePath, FileObjectR]
+PathOrFileW = Union[FilePath, FileObjectW]
 
 
 class Workbooks(QuerysetEndpoint):
@@ -117,12 +123,13 @@ class Workbooks(QuerysetEndpoint):
 
     # delete all the extracts on 1 workbook
     @api(version="3.3")
-    def delete_extract(self, workbook_item: WorkbookItem, includeAll: bool = True) -> None:
+    def delete_extract(self, workbook_item: WorkbookItem, includeAll: bool = True, datasources=None) -> JobItem:
         id_ = getattr(workbook_item, "id", workbook_item)
         url = "{0}/{1}/deleteExtract".format(self.baseurl, id_)
-        datasource_req = RequestFactory.Workbook.embedded_extract_req(includeAll, None)
+        datasource_req = RequestFactory.Workbook.embedded_extract_req(includeAll, datasources)
         server_response = self.post_request(url, datasource_req)
         new_job = JobItem.from_response(server_response.content, self.parent_srv.namespace)[0]
+        return new_job
 
     # Delete 1 workbook by id
     @api(version="2.0")
@@ -178,38 +185,11 @@ class Workbooks(QuerysetEndpoint):
     def download(
         self,
         workbook_id: str,
-        filepath: FilePath = None,
+        filepath: Optional[PathOrFileW] = None,
         include_extract: bool = True,
         no_extract: Optional[bool] = None,
     ) -> str:
-        if not workbook_id:
-            error = "Workbook ID undefined."
-            raise ValueError(error)
-        url = "{0}/{1}/content".format(self.baseurl, workbook_id)
-
-        if no_extract is False or no_extract is True:
-            import warnings
-
-            warnings.warn(
-                "no_extract is deprecated, use include_extract instead.",
-                DeprecationWarning,
-            )
-            include_extract = not no_extract
-
-        if not include_extract:
-            url += "?includeExtract=False"
-
-        with closing(self.get_request(url, parameters={"stream": True})) as server_response:
-            _, params = cgi.parse_header(server_response.headers["Content-Disposition"])
-            filename = to_filename(os.path.basename(params["filename"]))
-
-            download_path = make_download_path(filepath, filename)
-
-            with open(download_path, "wb") as f:
-                for chunk in server_response.iter_content(1024):  # 1KB
-                    f.write(chunk)
-        logger.info("Downloaded workbook to {0} (ID: {1})".format(download_path, workbook_id))
-        return os.path.abspath(download_path)
+        return self.download_revision(workbook_id, None, filepath, include_extract, no_extract)
 
     # Get all views of workbook
     @api(version="2.0")
@@ -250,7 +230,7 @@ class Workbooks(QuerysetEndpoint):
         logger.info("Populated connections for workbook (ID: {0})".format(workbook_item.id))
 
     def _get_workbook_connections(
-        self, workbook_item: WorkbookItem, req_options: "RequestOptions" = None
+        self, workbook_item: WorkbookItem, req_options: Optional["RequestOptions"] = None
     ) -> List[ConnectionItem]:
         url = "{0}/{1}/connections".format(self.baseurl, workbook_item.id)
         server_response = self.get_request(url, req_options)
@@ -259,7 +239,7 @@ class Workbooks(QuerysetEndpoint):
 
     # Get the pdf of the entire workbook if its tabs are enabled, pdf of the default view if its tabs are disabled
     @api(version="3.4")
-    def populate_pdf(self, workbook_item: WorkbookItem, req_options: "RequestOptions" = None) -> None:
+    def populate_pdf(self, workbook_item: WorkbookItem, req_options: Optional["RequestOptions"] = None) -> None:
         if not workbook_item.id:
             error = "Workbook item missing ID."
             raise MissingRequiredFieldError(error)
@@ -331,7 +311,7 @@ class Workbooks(QuerysetEndpoint):
     def publish(
         self,
         workbook_item: WorkbookItem,
-        file: PathOrFile,
+        file: PathOrFileR,
         mode: str,
         connection_credentials: Optional["ConnectionCredentials"] = None,
         connections: Optional[Sequence[ConnectionItem]] = None,
@@ -349,7 +329,6 @@ class Workbooks(QuerysetEndpoint):
             )
 
         if isinstance(file, (str, os.PathLike)):
-            # Expect file to be a filepath
             if not os.path.isfile(file):
                 error = "File path does not lead to an existing file."
                 raise IOError(error)
@@ -365,12 +344,12 @@ class Workbooks(QuerysetEndpoint):
                 error = "Only {} files can be published as workbooks.".format(", ".join(ALLOWED_FILE_EXTENSIONS))
                 raise ValueError(error)
 
-        elif isinstance(file, (io.BytesIO, io.BufferedReader)):
-            # Expect file to be a file object
-            file_size = get_file_object_size(file)
+        elif isinstance(file, io_types_r):
+            if not workbook_item.name:
+                error = "Workbook item must have a name when passing a file object"
+                raise ValueError(error)
 
             file_type = get_file_type(file)
-
             if file_type == "zip":
                 file_extension = "twbx"
             elif file_type == "xml":
@@ -379,13 +358,10 @@ class Workbooks(QuerysetEndpoint):
                 error = "Unsupported file type {}!".format(file_type)
                 raise ValueError(error)
 
-            if not workbook_item.name:
-                error = "Workbook item must have a name when passing a file object"
-                raise ValueError(error)
-
             # Generate filename for file object.
             # This is needed when publishing the workbook in a single request
             filename = "{}.{}".format(workbook_item.name, file_extension)
+            file_size = get_file_object_size(file)
 
         else:
             raise TypeError("file should be a filepath or file object.")
@@ -427,7 +403,7 @@ class Workbooks(QuerysetEndpoint):
                 with open(file, "rb") as f:
                     file_contents = f.read()
 
-            elif isinstance(file, (io.BytesIO, io.BufferedReader)):
+            elif isinstance(file, io_types_r):
                 file_contents = file.read()
 
             else:
@@ -488,14 +464,17 @@ class Workbooks(QuerysetEndpoint):
         self,
         workbook_id: str,
         revision_number: str,
-        filepath: Optional[PathOrFile] = None,
+        filepath: Optional[PathOrFileW] = None,
         include_extract: bool = True,
         no_extract: Optional[bool] = None,
-    ) -> str:
+    ) -> PathOrFileW:
         if not workbook_id:
             error = "Workbook ID undefined."
             raise ValueError(error)
-        url = "{0}/{1}/revisions/{2}/content".format(self.baseurl, workbook_id, revision_number)
+        if revision_number is None:
+            url = "{0}/{1}/content".format(self.baseurl, workbook_id)
+        else:
+            url = "{0}/{1}/revisions/{2}/content".format(self.baseurl, workbook_id, revision_number)
 
         if no_extract is False or no_extract is True:
             import warnings
@@ -511,17 +490,22 @@ class Workbooks(QuerysetEndpoint):
 
         with closing(self.get_request(url, parameters={"stream": True})) as server_response:
             _, params = cgi.parse_header(server_response.headers["Content-Disposition"])
-            filename = to_filename(os.path.basename(params["filename"]))
-
-            download_path = make_download_path(filepath, filename)
-
-            with open(download_path, "wb") as f:
+            if isinstance(filepath, io_types_w):
                 for chunk in server_response.iter_content(1024):  # 1KB
-                    f.write(chunk)
+                    filepath.write(chunk)
+                return_path = filepath
+            else:
+                filename = to_filename(os.path.basename(params["filename"]))
+                download_path = make_download_path(filepath, filename)
+                with open(download_path, "wb") as f:
+                    for chunk in server_response.iter_content(1024):  # 1KB
+                        f.write(chunk)
+                return_path = os.path.abspath(download_path)
+
         logger.info(
-            "Downloaded workbook revision {0} to {1} (ID: {2})".format(revision_number, download_path, workbook_id)
+            "Downloaded workbook revision {0} to {1} (ID: {2})".format(revision_number, return_path, workbook_id)
         )
-        return os.path.abspath(download_path)
+        return return_path
 
     @api(version="2.3")
     def delete_revision(self, workbook_id: str, revision_number: str) -> None:
